
/* Code copied from librnd and relicensed by the original author. */

#include <stdio.h>
#include <string.h>

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>


static prim_t *primbuf_add(glfw_win_t *gw, int type, GLsizei vx_count)
{
	prim_t *p = vtpr_alloc_append(&gw->prbuf, 1);
	p->type = type;
	p->vx_start = gw->vxbuf.used;
	p->vx_len = vx_count;

	/* use window current colors by default (caller can override after ret) */
	p->r = gw->red;
	p->g = gw->green;
	p->b = gw->blue;
	p->a = gw->alpha;

	/* no texture by default (caller can override after ret) */
	p->txtid = 0;

	return p;
}


static void vertbuf_add(glfw_win_t *gw, GLfloat x, GLfloat y)
{
	vertex_t *p_vert = vtvx_alloc_append(&gw->vxbuf, 1);
	p_vert->x = x;
	p_vert->y = y;
}

static void vertbuf_add_xyuv(glfw_win_t *gw, GLfloat x, GLfloat y, GLfloat u, GLfloat v)
{
	vertex_t *p_vert = vtvx_alloc_append(&gw->vxbuf, 1);
	p_vert->x = x;
	p_vert->y = y;
	p_vert->u = u;
	p_vert->v = v;
}

/* Upload vertex buffer buf; buf_size is the byte size of the whole buffer;
   the buffer is an array of elems of size elem_size. Each elem has an x,y
   coord pair at offset x_offs. */
static void begin_vertbuf(glfw_win_t *gw, void *buf, int comps, long buf_size, long elem_size, size_t x_offs)
{
	/* This is the buffer that holds the vertices */
	glBindBuffer(GL_ARRAY_BUFFER, gw->position_buffer);
	glBufferData(GL_ARRAY_BUFFER, buf_size, buf, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	/* Use the vertices in our buffer */
	glBindBuffer(GL_ARRAY_BUFFER, gw->position_buffer);
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, comps, GL_FLOAT, GL_FALSE, elem_size, (void *)x_offs);
}

static void end_vertbuf(glfw_win_t *gw)
{
}


/* Set drawing color of the following glDrawArrays() calls */
static void color_vertbuf(glfw_win_t *gw, float r, float g, float b, float a)
{
	if ((r != gw->vertbuf_last_r) || (g != gw->vertbuf_last_g) || (b != gw->vertbuf_last_b) || (a != gw->vertbuf_last_a)) {
		gw->vertbuf_last_r = r;
		gw->vertbuf_last_g = g;
		gw->vertbuf_last_b = b;
		gw->vertbuf_last_a = a;
		glUniform4f(gw->inputColor_location, r, g, b, a);
	}
}

static long texture_import(unsigned char *pixels, int width, int height, int has_alpha)
{
	GLuint texture_id;

	glGenTextures(1, &texture_id);
	glBindTexture(GL_TEXTURE_2D, texture_id);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, has_alpha ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, pixels);

	return texture_id;
}

static void texture_free(long id)
{
	GLuint tmp = id;
	glDeleteTextures(1, &tmp);
}

static void config_texture(GLuint texture_id)
{
	glBindTexture(GL_TEXTURE_2D, texture_id);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glEnable(GL_TEXTURE_2D);
}

static glfw_win_t *win2gw(mbtk_window_t *win)
{
	while(win->in_widget != NULL)
		win = win->in_widget->win;
	return (glfw_win_t *)win;
}

/* Set viewport and execute all draw from the vxbuf, reset vxbuf */
static void mbtk_glfw_draw_flush_(mbtk_window_t *win, glfw_win_t *gw)
{
	long n;
	int skip = 0;

	if (win->in_widget != NULL) {
		long w = win->in_widget->hvbox.w, h = win->in_widget->hvbox.h;
		long x = win->in_widget->hvbox.x, y = win->in_widget->win->h - win->in_widget->hvbox.y - h;
		glViewport(x, y, w, h);
		glUniform4f(gw->xform_location, 0, 0, 1.0/((double)w/2.0), -1.0/((double)h/2.0));
	}
	else {
		glViewport(0, 0, win->w, win->h);
		glUniform4f(gw->xform_location, 0, 0, 1.0/((double)win->w/2.0), -1.0/((double)win->h/2.0));
	}

	if (win->native_draw_pre != NULL)
		skip = win->native_draw_pre(win, NULL);

	if (!skip) {
		/* draw all primitives */
		begin_vertbuf(gw, gw->vxbuf.array, 4, gw->vxbuf.used * sizeof(vertex_t), sizeof(vertex_t), 0);
		for(n = 0; n < gw->prbuf.used; n++) {
			prim_t *p = &gw->prbuf.array[n];

			if (p->txtid > 0) {
				glActiveTexture(GL_TEXTURE0);
				config_texture(p->txtid);
				color_vertbuf(gw, -3.3, 0, 0, 0); /* enable using texture instead of color */
			}
			else
				color_vertbuf(gw, p->r, p->g, p->b, p->a);

/*printf("prim type=%d start=%d len=%d txt=%d\n", p->type, p->vx_start, p->vx_len, p->txtid);*/
			glDrawArrays(p->type, p->vx_start, p->vx_len);

			if (p->txtid > 0)
				glDisable(GL_TEXTURE_2D);
		}

/*	glFlush();
	glFinish();*/

		end_vertbuf(gw);
	}

	gw->vxbuf.used = 0;
	gw->prbuf.used = 0;
	gw->vertbuf_last_r = gw->vertbuf_last_g = gw->vertbuf_last_b = gw->vertbuf_last_a = -1;

	if (win->native_draw_post != NULL)
		win->native_draw_post(win, NULL);
}

static void mbtk_glfw_draw_flush(mbtk_window_t *win)
{
	glfw_win_t *gw = win2gw(win);

	mbtk_glfw_draw_flush_(win, gw);


	/* The last window flushed is the outmost window that's not in a widget;
	   when that happens, we have drawn everything so we can flush to screen */
	if (win->in_widget == NULL)
		glfwSwapBuffers(gw->window);
}


static void mbtk_glfw_draw_start(mbtk_window_t *win)
{
	glfw_win_t *gw = win2gw(win);
	int width, height;

	if (win->in_widget) {
		/* draw anything we already have in the buffer, because our new draw will be
		   on top of it */
		mbtk_glfw_draw_flush_(win->in_widget->win, gw);
		return; /* the real target window is already initialized */
	}

	gw->vertbuf_last_r = gw->vertbuf_last_g = gw->vertbuf_last_b = gw->vertbuf_last_a = -1;
	gw->vxbuf.used = 0;
	gw->prbuf.used = 0;

	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glDisable(GL_STENCIL_TEST);

	glUseProgram(gw->program);

	glfwGetFramebufferSize(gw->window, &width, &height);
	win->w = width;
	win->h = height;

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glClear(GL_COLOR_BUFFER_BIT);
}


static int gl_is_es(void)
{
	static const char es_prefix[] = "OpenGL ES";
	const char *ver;

	/* libepoxy says: some ES implementions (e.g. PowerVR) don't have the
	   es_prefix in their version string. For now we ignore these as
	   they need dlsym() to detect */

	ver = (const char *)glGetString(GL_VERSION);
	if (ver == NULL)
		return 0;

	return strncmp(ver, es_prefix, sizeof(es_prefix)-1) == 0;
}

static GLint gl_get_ver_major(void)
{
	GLint major = 0;

#ifdef GL_MAJOR_VERSION
	glGetIntegerv(GL_MAJOR_VERSION, &major);
#endif

	if (major == 0)
		glGetIntegerv(GL_VERSION, &major);

	if (major == 0) {
		const char *verstr = (const char *)glGetString(GL_VERSION);
		if ((verstr != NULL) && (strncmp(verstr, "OpenGL ES", 9) == 0)) {
			char *end;
			long tmp = strtol(verstr+10, &end, 10);
			if (*end == '.') {
				mbtk_warning("opengl gl_get_ver_major: had to extract verison from string: %ld from '%s'\n", tmp, verstr);
				return tmp;
			}
			else
				mbtk_warning("opengl gl_get_ver_major: tried to extract verison from string '%s' but failed the conversion; end='%s'\n", verstr, end);
		}
		mbtk_error("opengl gl_get_ver_major: you have a real ancient opengl version '%s'\n", verstr == NULL ? "<unknown>" : verstr);
		return -1;
	}

	return major;
}

static int init_checkver(void)
{
	GLint profmask = 0, major = gl_get_ver_major();

	if (major < 0) {
		mbtk_error("opengl draw: glfw/gl init refuse: failed to determine opengl version\n");
		return -1;
	}

	if (gl_is_es()) {
		if (major >= 1) {
			mbtk_error("opengl draw: glfw/gl init accept (ES with major %d)\n", major);
			return 0;
		}
	}

	if (major < 3) {
		mbtk_error("opengl draw: glfw/gl init refuse: major %d is below 3\n", major);
		return -1;
	}

#ifdef GL_CONTEXT_PROFILE_MASK
	glGetIntegerv(GL_CONTEXT_PROFILE_MASK, &profmask);
	if ((profmask != 0) && !(profmask & GL_CONTEXT_CORE_PROFILE_BIT)) {
		mbtk_error("opengl draw: glfw/gl init refuse: GL_CONTEXT_PROFILE_MASK (%d) lacks core mode in major %d\n", profmask, major);
		return -1;
	}
#else
	mbtk_error("opengl draw: glfw/gl init refuse: GL_CONTEXT_PROFILE_MASK missing with major %d\n", major);
	return -1;
#endif

/*	fprintf(stderr, "opengl draw: glfw/gl init accept\n");*/
	return 0;
}


/* Create and compile a shader */
static GLuint create_shader(int type, const char *src)
{
	int status;
	GLuint shader = glCreateShader(type);

	glShaderSource(shader, 1, &src, NULL);
	glCompileShader(shader);

	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (status == GL_FALSE) {
		int log_len;
		char *buffer;

		glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &log_len);

		buffer = malloc(log_len + 1);
		*buffer = 0;
		glGetShaderInfoLog(shader, log_len, NULL, buffer);
		mbtk_error("opengl draw: glfw/gl init: Compile failure in %s shader:\n%s\n", type == GL_VERTEX_SHADER ? "vertex" : "fragment", buffer);
		free(buffer);

		glDeleteShader(shader);
		return 0;
	}

	return shader;
}

/* Initialize the shaders and link them into a program */
static int init_shaders_(const char *vertex_sh, const char *fragment_sh, GLuint *program_out, GLuint *inputColor_out, GLuint *inputTexture_out, GLuint *xform_out)
{
	GLuint vertex, fragment;
	GLuint program = 0;
	GLuint inputColor = 0;
	GLuint inputTexture = 0;
	GLuint xform = 0;
	int status, res = -1;

	vertex = create_shader(GL_VERTEX_SHADER, vertex_sh);
	if (vertex == 0)
		return -1;

	fragment = create_shader(GL_FRAGMENT_SHADER, fragment_sh);
	if (fragment == 0) {
		glDeleteShader(vertex);
		return -1;
	}

	program = glCreateProgram();
	glAttachShader(program, vertex);
	glAttachShader(program, fragment);

	glLinkProgram(program);

	glGetProgramiv(program, GL_LINK_STATUS, &status);
	if (status == GL_FALSE) {
		int log_len;
		char *buffer;

		glGetProgramiv(program, GL_INFO_LOG_LENGTH, &log_len);

		buffer = malloc(log_len + 1);
		glGetProgramInfoLog(program, log_len, NULL, buffer);
		mbtk_error("opengl draw: glfw/gl init: Linking failure:\n%s\n", buffer);
		free(buffer);

		glDeleteProgram(program);
		program = 0;

		goto out;
	}

	inputColor = glGetUniformLocation(program, "inputColor");
	inputTexture = glGetUniformLocation(program, "inputTexture");
	xform = glGetUniformLocation(program, "xform");

	glDetachShader(program, vertex);
	glDetachShader(program, fragment);
	res = 0;

out:
	glDeleteShader(vertex);
	glDeleteShader(fragment);

	*program_out = program;
	*inputColor_out = inputColor;
	*inputTexture_out = inputTexture;
	*xform_out = xform;

	return res;
}

static int init_shaders(glfw_win_t *gw)
{
	const char *vertex_sh, *fragment_sh;

#define NL "\n"

	if (gl_is_es()) {
/*		fprintf(stderr, "opengl draw: init_shaders: opengl ES\n");*/
		vertex_sh = 
			NL "attribute vec4 position;"
			NL "varying vec2 TexCoord;"
			NL "uniform vec4 xform;"
			NL "void main() {"
			NL "  gl_Position = vec4((position[0] + xform[0]) * xform[2] - 1.0, (position[1] + xform[1]) * xform[3] + 1.0, 0, 1);"
			NL "  TexCoord = vec2(position[2], position[3]);"
			NL "}"
			NL;

		fragment_sh =
			NL "precision highp float;"
			NL "varying vec2 TexCoord;"
			NL "uniform vec4 inputColor;"
			NL "uniform sampler2D inputTexture;" /* we do not have to load this one, as 0 is default */
			NL "void main() {"
			NL "  if ((inputColor[0] < -3.0)) {"
			NL "    gl_FragColor = texture2D(inputTexture, TexCoord);"
			NL "  } else {"
			NL "    gl_FragColor = inputColor;"
			NL "  }"
			NL "}"
			NL;
	}
	else {
/*		fprintf(stderr, "opengl draw: init_shaders: opengl desktop\n");*/
		vertex_sh = 
			NL "#version 330"
			NL "attribute vec4 position;"
			NL "out vec2 TexCoord;"
			NL "uniform vec4 xform;"
			NL "void main() {"
			NL "  gl_Position = vec4((position[0] + xform[0]) * xform[2] - 1.0, (position[1] + xform[1]) * xform[3] + 1.0, 0, 1);"
			NL "  TexCoord = vec2(position[2], position[3]);"
			NL "}"
			NL ";"
			NL;

		fragment_sh =
			NL "#version 330"
			NL "out vec4 outputColor;"
			NL "in vec2 TexCoord;"
			NL "uniform vec4 inputColor;"
			NL "uniform sampler2D inputTexture;" /* we do not have to load this one, as 0 is default */
			NL "void main() {"
			NL "  if (inputColor[0] < -3) {"
			NL "    outputColor = texture(inputTexture, TexCoord);"
			NL "  } else {"
			NL "    outputColor = inputColor;"
			NL "  }"
			NL "}"
			NL;
	}


	return init_shaders_(vertex_sh, fragment_sh, &gw->program, &gw->inputColor_location, &gw->inputTexture_location, &gw->xform_location);
}

static int draw_win_init(glfw_win_t *gw)
{
	GLuint vao;

	if (init_shaders(gw) != 0) {
		mbtk_error("opengl draw: draw_win_init: failed to init shaders, no rendering is possible\n");
		return -1;
	}

	/* We only use one VAO, so we always keep it bound */
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);
	glGenBuffers(1, &gw->position_buffer);

	return 0;
}

static void draw_win_uninit(glfw_win_t *gw)
{
	glDeleteProgram(gw->program);
	glDeleteBuffers(1, &gw->position_buffer);
	vtvx_uninit(&gw->vxbuf);
	vtpr_uninit(&gw->prbuf);
}

