#ifdef MBTK_HAVE_SDL2_TICKS64
static double mbtk_sdl2_get_ticks(void)
{
	Uint64 ticks = SDL_GetTicks64();
	return (double)ticks / 1000.0;
}
#else
/* workaround: use the 32 bit variant and compensate for the 49-day rollover */
static double mbtk_sdl2_get_ticks(void)
{
	static Uint32 last_ticks = 0; /* no need to store these in display as SDL has only one, central tick */
	static double rollovers = 0;
	Uint32 ticks = SDL_GetTicks();
	double res;

	res = (double)ticks / 1000.0;

	if (ticks < last_ticks)
		rollovers += 4294967.296;

	last_ticks = ticks;
	return res + rollovers;
}
#endif

static double mbtk_sdl2_get_time_el(mbtk_display_t *disp)
{
	sdl2_display_t *sd = (sdl2_display_t *)disp;
	return mbtk_sdl2_get_ticks() - sd->base_time;
}

static int mbtk_sdl2_display_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char **argv[])
{
	sdl2_display_t *sd = (sdl2_display_t *)disp;

	mbtk_be_check_priv_sizes(sdl2_display_t, sdl2_win_t);

	mbtk_sdl2_font_init(&sd->font_cache);
	memset(&sd->iot_fdwatch, 0, sizeof(sd->iot_fdwatch));
	sd->base_time = mbtk_sdl2_get_ticks();
	return 0;
}

static void mbtk_sdl2_watch_fd_uninit(mbtk_display_t *disp);
static void mbtk_sdl2_watch_stdin_uninit(mbtk_display_t *disp);

static int mbtk_sdl2_display_uninit(mbtk_backend_t *be, mbtk_display_t *disp)
{
	sdl2_display_t *sd = (sdl2_display_t *)disp;
	mbtk_sdl2_font_uninit(&sd->font_cache);
	mbtk_sdl2_watch_fd_uninit(disp);
	mbtk_sdl2_watch_stdin_uninit(disp);
	return 0;
}

static mbtk_keymod_t mbtk_sdl2_query_kbd_mods(mbtk_display_t *disp)
{
	mbtk_keymod_t res = 0;
	SDL_Keymod km = SDL_GetModState();

	if ((km & KMOD_LSHIFT) || (km & KMOD_RSHIFT))
		res |= MBTK_KM_SHIFT;
	if ((km & KMOD_LCTRL) || (km & KMOD_RCTRL))
		res |= MBTK_KM_CTRL;
	if ((km & KMOD_LALT) || (km & KMOD_RALT))
		res |= MBTK_KM_ALT;

	return res;
}
