#include <stdlib.h>
#include <string.h>
#include <libmbtk/util.h>
#include "../common/fdwatch.h"
#include "../common/pipes.h"

#include "../sdl2.h"


#include "mbtk_sdl2_priv.h"
static Uint32 mbtk_sdl2_ev_rstdio,  mbtk_sdl2_ev_fdwatch;

static int mbtk_sdl2_be_init(mbtk_backend_t *be)
{
	SDL_SetHint(SDL_HINT_NO_SIGNAL_HANDLERS, "1"); /* from 2.0.4: don't mess with SIGINT */
	SDL_Init(SDL_INIT_VIDEO);
	mbtk_sdl2_ev_rstdio = SDL_RegisterEvents(1);
	mbtk_sdl2_ev_fdwatch = SDL_RegisterEvents(1);
	return 0;
}

static void mbtk_sdl2_be_uninit(mbtk_backend_t *be)
{
	SDL_Quit();
}

#include "font.c"
#include "input_xlate.c"
#include "display.c"
#include "draw.c"
#include "io_thread.c"
#include "cursor.c"
#include "iterate.c"
#include "window.c"
#include "clipboard.c"

int mbtk_sdl2_backend(mbtk_backend_t *dst)
{
	memset(dst, 0, sizeof(mbtk_backend_t));

	dst->be_init = mbtk_sdl2_be_init;
	dst->be_uninit = mbtk_sdl2_be_uninit;
	dst->display_init = mbtk_sdl2_display_init;
	dst->display_uninit = mbtk_sdl2_display_uninit;
	dst->clipboard_set = mbtk_sdl2_clipboard_set;
	dst->clipboard_get = mbtk_sdl2_clipboard_get;
	dst->clipboard_free = mbtk_sdl2_clipboard_free;
	dst->iterate = mbtk_sdl2_iterate;
	dst->get_time_el = mbtk_sdl2_get_time_el;

	dst->window_new = mbtk_sdl2_window_new;
	dst->subwindow_new = mbtk_sdl2_subwindow_new;
	dst->window_close = mbtk_sdl2_window_close;
	dst->win2render_win = mbtk_sdl2_win2render_win;
	dst->window_set_title = mbtk_sdl2_window_set_title;
	dst->window_raise = mbtk_sdl2_window_raise;
	dst->window_hide = mbtk_sdl2_window_hide;
	dst->window_show = mbtk_sdl2_window_show;
	dst->window_transient4 = mbtk_sdl2_window_transient4;
	dst->window_set_min_size = mbtk_sdl2_window_set_min_size;
	dst->window_set_size = mbtk_sdl2_window_set_size;
	dst->window_get_size = mbtk_sdl2_window_get_size;

	dst->draw_color = mbtk_sdl2_draw_color;
	dst->draw_fillrect = mbtk_sdl2_draw_fillrect;
	dst->draw_line = mbtk_sdl2_draw_line;
	dst->draw_hline = mbtk_sdl2_draw_hline;
	dst->draw_vline = mbtk_sdl2_draw_vline;
	dst->draw_flush = mbtk_sdl2_draw_flush;
	dst->draw_start = mbtk_sdl2_draw_start;

	dst->resolve_font = mbtk_sdl2_resolve_font;
	dst->draw_set_font = mbtk_sdl2_draw_set_font;
	dst->draw_query_text = mbtk_sdl2_draw_query_text;
	dst->draw_text = mbtk_sdl2_draw_text;
	dst->draw_free_text = mbtk_sdl2_draw_free_text;

	dst->draw_import_pixmap = mbtk_sdl2_draw_import_pixmap;
	dst->draw_free_pixmap = mbtk_sdl2_draw_free_pixmap;
	dst->draw_bind_pixmap = mbtk_sdl2_draw_bind_pixmap;
/*	dst->draw_unbind_pixmap = mbtk_sdl2_draw_unbind_pixmap;*/
	dst->draw_pixmap = mbtk_sdl2_draw_pixmap;

	dst->mocu_create_named = mbtk_sdl2_mocu_create_named;
	dst->mocu_create_pixmap = mbtk_sdl2_mocu_create_pixmap;
	dst->mocu_free_cursor = mbtk_sdl2_mocu_free_cursor;
	dst->mocu_set_cursor = mbtk_sdl2_mocu_set_cursor;

	dst->query_kbd_mods = mbtk_sdl2_query_kbd_mods;

	dst->watch_stdin_read = mbtk_sdl2_watch_stdin_read;
	dst->watch_fd = mbtk_sdl2_watch_fd;

	dst->has_alpha = 1;

	return 0;
}
