#include <ctype.h>

#define need(min) \
do { \
	if (argc < min) { \
		fprintf(stderr, "Syntax error: %s requires at least %d arguments\n", cmd, min); \
		return 0; \
	} \
} while(0)

/* Execute cmd with argc/argv;
   if it emits an event, set dst and return 1;
   if it returns a string instead, strdup it and put it in *res (if res
   is not NULL) and return 0. */
static int eval_cmd(test_display_t *td, mbtk_event_t *dst, char *cmd, int argc, char *argv[], char **res)
{
	int n;

	if (strcmp(cmd, "echo") == 0) {
		for(n = 0; n < argc; n++)
			printf("%s ", argv[n]);
		printf("\n");
		return 0;
	}

	if ((strcmp(cmd, "key_press") == 0) || (strcmp(cmd, "key_release") == 0)) {
		char *keyname;
		int press = (cmd[4] == 'p');
		need(1);

		keyname = argv[0];

		dst->type = press ? MBTK_EV_KEY_PRESS : MBTK_EV_KEY_RELEASE;
		if (strcmp(keyname, "shift") == 0) {
			if (press) td->keymods |= MBTK_KM_SHIFT; else td->keymods &= ~MBTK_KM_SHIFT;
			dst->data.key.sym = MBTK_KS_SHIFT_L;
		}
		else if (strcmp(keyname, "ctrl") == 0) {
			if (press) td->keymods |= MBTK_KM_CTRL; else td->keymods &= ~MBTK_KM_CTRL;
			dst->data.key.sym = MBTK_KS_CTRL_L;
		}
		else if (strcmp(keyname, "alt") == 0) {
			if (press) td->keymods |= MBTK_KM_ALT; else td->keymods &= ~MBTK_KM_ALT;
			dst->data.key.sym = MBTK_KS_ALT_L;
		}
		else
			dst->data.key.sym = dst->data.key.edit = keyname[0];

		dst->data.key.mods = td->keymods;
		return 1;
	}

	printf("UNKNOWN CMD: '%s' %d res=%p\n", cmd, argc, res);
	for(n = 0; n < argc; n++)
		printf(" '%s'\n", argv[n]);

	return 0;
}

static int eval_cmdline(test_display_t *td, mbtk_event_t *dst, char *line, char **res, char **end)
{
	char *as, *s, *par, *argv[32], *tmp[32];
	int argc = 0, tmpc = 0, retv, n, is_par = 0, in_dq = 0;

	while(isspace(*line)) line++;

	/* skip emtpy lines */
	if (*line == '\0')
		return 0;

	par = strchr(line, '(');

	if (par == NULL) {
		/* the whole line is the command, no arguments */
		return eval_cmd(td, dst, line, 0, NULL, res);
	}

	*par = '\0';
	par++;

	/* parse arguments */
	for(as = s = par; ((*s != '\0') && !is_par); s++) {
		if (in_dq) {
			if (*s == '"') {
				*s = '\0';
				in_dq = 0;
			}
		}
		else
		switch(*s) {
			case '"':
				if (as == s) {
					as++;
					in_dq = 1;
				}
				break;
			case ')':
				is_par = 1;
			case ',': /* next arg */
				*s = '\0';
				argv[argc++] = as;
				as = s+1;
				while(*as == ' ') as++;
				break;
			case '(':
				tmp[tmpc] = NULL;
				eval_cmdline(td, NULL, as, &tmp[tmpc], &s);
				if (tmp[tmpc] != NULL) {
					as = tmp[tmpc];
					tmpc++;
				}
				else
					as = "";
				break;
		}
	}

	if (end != NULL)
		*end = s-1;

	retv = eval_cmd(td, dst, line, argc, argv, res);

	for(n = 0; n < tmpc; n++)
		free(tmp[n]);

	return retv;
}


static int mbtk_test_proc_event(test_display_t *td, mbtk_event_t *dst)
{
	char line[1024];

	*line = '\0';
	fgets(line, sizeof(line), stdin);
	return eval_cmdline(td, dst, line, NULL, NULL);
}

static int mbtk_test_iterate(mbtk_display_t *display, mbtk_event_t *dst, double timeout, int block)
{
	test_display_t *td = (test_display_t *)display;
	int sr;
	long usec;

	do {
		usec = timeout * 1000000.0;
		sr = mbtk_watch_fd_select(&td->watch, dst, &usec, display->topwin);
		if (sr < 0) {
			perror("select() error");
			return -1;
		}
		if (sr > 0) {
			/* our only watch is stdin for commands */
			if (mbtk_test_proc_event(td, dst))
				return 1;
		}
	} while(block);

	return 0;
}
