#include <X11/cursorfont.h>

struct mbtk_mocu_s {
	Cursor c;
	Pixmap pixel, mask;
	unsigned has_pix:1;
};

static mbtk_mocu_t *mbtk_xlib_mocu_create_named(mbtk_display_t *disp, mbtk_mocu_named_t name)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	mbtk_mocu_t *res;
	int shape;

	switch(name) {
		case MBTK_MOCU_ARROW:     shape = XC_top_left_arrow; break;
		case MBTK_MOCU_IBEAM:     shape = XC_xterm; break;
		case MBTK_MOCU_CROSSHAIR: shape = XC_crosshair; break;
		case MBTK_MOCU_HAND:      shape = XC_hand1; break;
		case MBTK_MOCU_HARROW:    shape = XC_sb_h_double_arrow; break;
		case MBTK_MOCU_VARROW:    shape = XC_sb_v_double_arrow; break;
		default: return NULL;
	}

	res = calloc(sizeof(mbtk_mocu_t), 1);
	res->c = XCreateFontCursor(xd->display, shape);
	return res;
}

static mbtk_mocu_t *mbtk_xlib_mocu_create_pixmap(mbtk_display_t *disp, int width, int height, unsigned const char *px, unsigned const char *mask, int hot_x, int hot_y)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	xlib_win_t *xw = (xlib_win_t *)disp->topwin;
	mbtk_mocu_t *res;
	XColor fg, bg;


	res = calloc(sizeof(mbtk_mocu_t), 1);
	res->pixel = XCreateBitmapFromData(xd->display, xw->window, (const char *)px, width, height);
	res->mask = XCreateBitmapFromData(xd->display, xw->window, (const char *)mask, width, height);
	res->has_pix = 1;

	fg.red = fg.green = fg.blue = 65535;
	bg.red = bg.green = bg.blue = 0;
	fg.flags = bg.flags = DoRed | DoGreen | DoBlue;
	res->c = XCreatePixmapCursor(xd->display, res->pixel, res->mask, &fg, &bg, 0, 0);

	return res;
}

static void mbtk_xlib_mocu_free_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	XFreeCursor(xd->display, curs->c);
	if (curs->has_pix) {
		XFreePixmap(xd->display, curs->pixel);
		XFreePixmap(xd->display, curs->mask);
	}
	free(curs);
}

static void mbtk_xlib_mocu_set_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs)
{
	xlib_display_t *xd = (xlib_display_t *)disp;
	xlib_win_t *xw = (xlib_win_t *)disp->topwin;
	XDefineCursor(xd->display, xw->window, curs->c);
	XFlush(xd->display);
}


