struct motif_hint_s {
	unsigned long flags, functions, decorations;
	long input_mode;
	unsigned long status;
};

static void mbtk_xlib_win_border(xlib_win_t *win, int want_border)
{
	mbtk_window_t *mwin = (mbtk_window_t *)win;
	xlib_display_t *xd = (xlib_display_t *)mwin->display;

	/* SDL2 implementation says _MOTIF_WM_HINTS is respected by most window
	   managers these days */
	if (xd->xa.wm_hints != None) { /* motif hint is available, use that */
		struct motif_hint_s motif_hint = { (1L << 1), 0, 0, 0, 0 };
		motif_hint.decorations = (want_border ? 1 : 0);
		XChangeProperty(xd->display, win->window, xd->xa.wm_hints, xd->xa.wm_hints, 32,
			PropModeReplace, (unsigned char *)&motif_hint,
			sizeof(motif_hint) / sizeof(long));
	}
	else  /* fallback if there's no motif compatibilirt: transient hint */
		XSetTransientForHint(xd->display, win->window, RootWindow(xd->display, xd->screen));
}

static int mbtk_xlib_win_wm_type(xlib_win_t *win, Atom awmt)
{
	mbtk_window_t *mwin = (mbtk_window_t *)win;
	xlib_display_t *xd = (xlib_display_t *)mwin->display;

	return XChangeProperty(xd->display, win->window, xd->xa.wintype, XA_ATOM, 32,
		PropModeReplace, (unsigned char *)&awmt, 1);
}

static int mbtk_xlib_win_wm_state(xlib_win_t *win, Atom awms)
{
	mbtk_window_t *mwin = (mbtk_window_t *)win;
	xlib_display_t *xd = (xlib_display_t *)mwin->display;

	return XChangeProperty(xd->display, win->window, xd->xa.wmstate, XA_ATOM, 32,
		PropModeReplace, (unsigned char *)&awms, 1);
}

static void mbtk_xlib_win_transient4_(xlib_win_t *win, xlib_win_t *tr_for)
{
	mbtk_window_t *mwin = (mbtk_window_t *)win;
	xlib_display_t *xd = (xlib_display_t *)mwin->display;
	XSetTransientForHint(xd->display, win->window, tr_for->window);
}

void mbtk_xlib_window_set_type(xlib_win_t *win, mbtk_window_type_t type)
{
	mbtk_window_t *mwin = (mbtk_window_t *)win;
	xlib_display_t *xd = (xlib_display_t *)mwin->display;

	switch(type & 0xff) {
		case MBTK_WNTY_NORMAL:
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_dialog);
			break;
		case MBTK_WNTY_MODAL:
/*			if (mwin->display->topwin != NULL)
				mbtk_xlib_win_transient4_(win, mwin->display->topwin);*/
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_dialog);
			mbtk_xlib_win_wm_state(win, xd->xa.wmst_modal);
			break;
		case MBTK_WNTY_DROPDOWN_MENU:
			mbtk_xlib_win_border(win, 0);
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_ddmenu);
			break;
		case MBTK_WNTY_POPUP_MENU:
			mbtk_xlib_win_border(win, 0);
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_pumenu);
			break;
		case MBTK_WNTY_TEAROFF_MENU:
			mbtk_xlib_win_border(win, 0);
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_menu);
			break;
		case MBTK_WNTY_COMBO:
			mbtk_xlib_win_border(win, 0);
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_combo);
			break;
		case MBTK_WNTY_TOOLTIP:
			mbtk_xlib_win_border(win, 0);
			mbtk_xlib_win_wm_type(win, xd->xa.wtype_ttip);
			break;
	}
}

