#include <stdio.h>
#include <libmbtk/global_typedefs.h>
#include <libmbtk/handler.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>

#include "hand_hovradio.h"

static const char my_name[] = "hand_hovradio";

void mbtk_hand_hovradio_activated(mbtk_widget_t *widget)
{
	mbtk_hand_radio_activated(widget);
}


static mbtk_event_handled_t hovradio_event(mbtk_widget_t *widget, mbtk_event_t *ev)
{
	mbtk_handler_radio_t *hr = widget->handler_data;

	assert(widget->handler->name == my_name);

	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
			if (ev->data.key.sym == MBTK_KS_LEFT) {
			}
			else if (ev->data.key.sym == MBTK_KS_RIGHT) {
			}
			break;
		case MBTK_EV_MOUSE_PRESS:
			if ((ev->data.mouse.button == MBTK_MB_LEFT) && !widget->flags.activated) {
				mbtk_hand_radio_activated(widget);
				mbtk_window_set_focus(widget->win, widget, 1);
				return MBTK_EVENT_HANDLED;
			}
			break;

		case MBTK_EV_MOUSE_MOTION:
			if ((hr->active >= 0) && !widget->flags.activated) {
				mbtk_hand_radio_activated(widget);
				mbtk_window_set_focus(widget->win, widget, 1);
				return MBTK_EVENT_HANDLED;
			}
			break;

		case MBTK_EV_FOCUS_MOUSE_PRESS_AWAY:
			if (widget->flags.activated) {
				mbtk_kw_t kw_toggled;

				mbtk_kw_cache(kw_toggled, "toggled");

				mbtk_widget_set_activated(widget, 0);
				mbtk_callback_noarg(widget, kw_toggled);
			}

			hr->active = -1;
			break;

		default:;
	}
	return MBTK_EVENT_NOT_HANDLED;
}

static void hovradio_widget_remove(mbtk_widget_t *widget)
{
	mbtk_handler_radio_t *hr = widget->handler_data;

	assert(widget->handler->name == my_name);

	mbtk_handler_hovradio_del_widget(hr, widget);
}

static void hovradio_handler_destroy(mbtk_handler_t *hr)
{
	mbtk_handler_radio_free((mbtk_handler_radio_t *)hr);
}



void mbtk_handler_hovradio_init(mbtk_window_t *parent, mbtk_handler_radio_t *hr)
{
	memset(hr, 0, sizeof(mbtk_handler_hovradio_t));

	hr->hdr.name = my_name;
	hr->hdr.event = hovradio_event;
	hr->hdr.widget_remove = hovradio_widget_remove;
	hr->hdr.handler_destroy = hovradio_handler_destroy;

	hr->active = -1;

	mbtk_window_add_handler(parent, &hr->hdr);
}


mbtk_handler_hovradio_t *mbtk_handler_hovradio_alloc(mbtk_window_t *parent)
{
	mbtk_handler_radio_t *hr;

	hr = malloc(sizeof(mbtk_handler_hovradio_t));
	mbtk_handler_hovradio_init(parent, hr);
	hr->alloced = 1;
	return hr;
}

void mbtk_handler_hovradio_uninit(mbtk_handler_hovradio_t *hr)
{
	mbtk_handler_radio_uninit(hr);
}

void mbtk_handler_hovradio_free(mbtk_handler_hovradio_t *hr)
{
	mbtk_handler_radio_free(hr);
}


void mbtk_handler_hovradio_add_widget(mbtk_handler_hovradio_t *hr, mbtk_widget_t *w)
{
	mbtk_handler_radio_add_widget(hr, w);
}

void mbtk_handler_hovradio_del_widget(mbtk_handler_hovradio_t *hr, mbtk_widget_t *w)
{
	mbtk_handler_radio_del_widget_nochk(hr, w);
}

