#ifndef LIBMBTK_HANDLER_RADIO_H
#define LIBMBTK_HANDLER_RADIO_H

#include <libmbtk/src_3rd/genvector/vtp0.h>
#include <libmbtk/handler.h>

typedef struct mbtk_handler_radio_s { /* dynamic allocated per radio button handler (widget group); acts as both handler and handler_data in widgets */
	mbtk_handler_t hdr;
	int active; /* index of the active widget or -1 if none activated */
	vtp0_t widgets;
	unsigned alloced:1; /* whether created using mbtk_handler_radio_alloc() */
} mbtk_handler_radio_t;


/* _alloc() uses malloc() and calls _init(); _init() is useful for
   static allocation. The handler is registered in a parent window and is
   destroyed when the parent window is destroyed. */
mbtk_handler_radio_t *mbtk_handler_radio_alloc(mbtk_window_t *parent);
void mbtk_handler_radio_init(mbtk_window_t *parent, mbtk_handler_radio_t *hr);

/* _free() is the pair of _alloc() and calls free(hr) if hr was allocated using
   mbtk_handler_radio_alloc(); _uninit() is the pair of _init() and frees
   only the fields of hr */
void mbtk_handler_radio_free(mbtk_handler_radio_t *hr);
void mbtk_handler_radio_uninit(mbtk_handler_radio_t *hr);


/* Append or remove a radi button; order of appneding shouldn't matter */
void mbtk_handler_radio_add_widget(mbtk_handler_radio_t *hr, mbtk_widget_t *w);
void mbtk_handler_radio_del_widget(mbtk_handler_radio_t *hr, mbtk_widget_t *w);


/*** For internal use ***/
void mbtk_hand_radio_activated(mbtk_widget_t *widget);
void mbtk_handler_radio_del_widget_nochk(mbtk_handler_radio_t *hr, mbtk_widget_t *w);

#endif
