#ifndef LIBMBTK_WIDGET_SCROLLBAR_H
#define LIBMBTK_WIDGET_SCROLLBAR_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Button: can be clicked a mouse click release or enter release */
typedef struct mbtk_scrollbar_s {
	mbtk_widget_t w;
	double min, max;
	double curr;          /* interpreted only between min and max (value copied and clamped) */
	double len;           /* length of the curr window */
	int thickness;        /* in pixels in the direction perpendicular to the main direction (see ->horiz) */
	unsigned int horiz:1; /* set to 1 for horizontal layout, 0 for vertical */
} mbtk_scrollbar_t;

/* Create a new scrollress bar with min/max set to 0/1, curr set to 0. */
mbtk_scrollbar_t *mbtk_scrollbar_new(mbtk_scrollbar_t *caller_alloced, int horiz);

/* Set curr value of prg. If curr is between min and max, inclusive, it is
set. If curr is out of range and clamp is true, it is clamped and set. Else
error is returned. Returns 0 on success, -1 on error. */
int mbtk_scrollbar_set_curr(mbtk_scrollbar_t *prg, double curr, double len, int clamp);

/* Bind the range.moved callback, which is called whenever the user (attempted)
   to change scrollbar curr position */
void mbtk_scrollbar_callback_range_moved(mbtk_scrollbar_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data);

MBTK_WIDGET_INLINES(scrollbar)

#endif
