#ifndef LIBMBTK_WIDGET_TBUTTON_H
#define LIBMBTK_WIDGET_TBUTTON_H

#include <libmbtk/config.h>
#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Toggle button: has two states, depressed or released; switched between
   the two by a mouse click release or enter release */
typedef struct mbtk_tbutton_s {
	mbtk_widget_t w;
} mbtk_tbutton_t;

/* Create a new tbutton with no child widget. The caller shall create a widget
   (e.g. a hbox with further widgets) and use mbtk_tbutton_set_child() to put
   install it in the tbutton. Handler is set to hand_psuhb. */
mbtk_tbutton_t *mbtk_tbutton_new(mbtk_tbutton_t *caller_alloced);

/* Create a new tbutton with a label installed in it. Note: always dynamically
   alloced because of the child label widget needs to be alloced. Use
   mbtk_tbutton_new_with_widget() for static allocation. */
mbtk_tbutton_t *mbtk_tbutton_new_with_label(const char *str);

/* Create a new tbutton with a widget installed in it. Can be used with static
   allocation, for example:
   mbtk_tbutton_new_with_widget(&w1, mbtk_label_new(&w2, "foo")); */
mbtk_tbutton_t *mbtk_tbutton_new_with_widget(mbtk_tbutton_t *caller_alloced, mbtk_widget_t *child);


/* A tbutton hosts a single widget (which can be a container, e.g. hbox or vbox).
   If the tbutton already had a child widget, it is unlinked (removed from the
   tbutton) but not freed.
   Returns original child on success (may be NULL). */
mbtk_widget_t *mbtk_tbutton_set_child(mbtk_tbutton_t *parent, mbtk_widget_t *w);


/* Called after the release on a click on the tbutton (that actually toggled
   the button right before the callback) */
void mbtk_tbutton_callback_toggle(mbtk_tbutton_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data);


MBTK_WIDGET_INLINES(tbutton)

#endif
