#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include <libmbtk/mpm.h>
#include "../be_init.h"

static char **font;
static int stay;

static const char *xpm_test[] = {
	"10 10 3 1",
	"o c #ff0000",
	"+ c #7777ff",
	". c None",
	".......+..",
	"......oo..",
	".....o+o .",
	"....o++o..",
	"...o+++o..",
	"..o++++o..",
	"...o+++o..",
	"....o++o..",
	".....oo...",
	"......+...",
};


static mbtk_event_handled_t button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Button click: '%s'\n", (const char *)user_data);
	return 0;
}

static mbtk_event_handled_t button_toggle(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Toggle: '%s' %d\n", (const char *)user_data, w->flags.activated);
	return 0;
}

static mbtk_event_handled_t range_moved(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Range moved\n");
	return 0;
}

mbtk_event_handled_t box_event_cb(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data)
{
	if (ev->type == MBTK_EV_KEY_PRESS)
		fprintf(stderr, "unhandled key press on %s\n", (char *)user_data);
	return 0;
}

static int canvas_minsize(mbtk_canvas_t *c, long *mx, long *my)
{
	*mx = *my = 25;
printf("canvas minsize\n");
	return 0;
}

static int canvas_draw(mbtk_canvas_t *c, long x_offs, long y_offs, long w, long h)
{
	mbtk_window_t *win = c->subwin;
printf("canvas draw\n");

	mbtk_draw_color(win, 0, 0, 255, 128);
	mbtk_draw_fillrect(win, 0, 0, 800, 222);

	mbtk_draw_color(win, 255, 0, 64, 255);
	mbtk_draw_line(win, 2, 2, 222, 222);

	return 0;
}

static mbtk_event_handled_t canvas_ev(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data)
{
	printf("canvas ev on %s: ", (char *)user_data);
	if (ev->type == MBTK_EV_MOUSE_MOTION) {
		printf("%ld %ld\n", ev->data.mouse.x, ev->data.mouse.y);
	}
	else
		printf("\n");
	return MBTK_EVENT_HANDLED;
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *vbox, *w1, *w2, *hbox1;
	mbtk_label_t *wl, *bl;
	mbtk_button_t *wb;
	mbtk_tbutton_t *wtb;
	mbtk_checkbox_t *wch;
	mbtk_radiobutton_t *wrb;
	mbtk_separator_t *wsep;
	mbtk_handler_radio_t *hrr;
	mbtk_progbar_t *wprb;
	mbtk_scrollbar_t *wscb;
	mbtk_picture_t *wpic;
	mbtk_kw_t pickw;
	mbtk_canvas_t *wcs;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);


	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

/*	mbtk_style_set_prop(disp.style, disp.styleprop, mbtk_kw("frame.width"), mbtk_arg_long(2));
	mbtk_style_set_prop(disp.style, disp.styleprop, mbtk_kw("frame.width"), mbtk_arg_long_mult(3));*/

/*	mbtk_display_style_set_prop(&disp, mbtk_kw("frame.width"), mbtk_arg_long(2));
	mbtk_display_style_set_prop(&disp, mbtk_kw("frame.width"), mbtk_arg_long_mult(3));*/

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk simple widgets", -1, -1, 300, 300);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	vbox = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &vbox->w);

	w1 = mbtk_vbox_new(NULL);
	w2 = mbtk_vbox_new(NULL);
	wl = mbtk_label_new(NULL, "hello\nworld!");
	bl = mbtk_label_new(NULL, "click me");
	wb = mbtk_button_new(NULL);
	mbtk_button_set_child(wb, &bl->w);
	mbtk_button_callback_click(wb, button_clk, "blobb");

	wtb = mbtk_tbutton_new_with_label("toggle me");
	mbtk_tbutton_callback_toggle(wtb, button_toggle, "buttonbarf");

	mbtk_box_add_widget(vbox, &w1->w, 0);
	mbtk_box_add_widget(vbox, &wl->w, 0);
	mbtk_box_add_widget(vbox, &wb->w, 0);
	mbtk_box_add_widget(vbox, &wtb->w, 0);

	/*** Checkbox button ***/

	/* create and append a hbox; this will host our example radio buttons */
	hbox1 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	mbtk_handler_ev2cb_install(&hbox1->w);
	mbtk_box_callback_event(hbox1, box_event_cb, "checkbox hbox");


	/* create a checkbox; preserve default handler (which is "toggle") */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf1");
	mbtk_box_add_widget(hbox1, &wch->w, 0);

	/* create another checkbox */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf2");
	mbtk_box_add_widget(hbox1, &wch->w, 0);


	/* create another checkbox */
	wch = mbtk_checkbox_new(NULL);
	mbtk_checkbox_callback_toggle(wch, button_toggle, "chkbarf3");
	mbtk_box_add_widget(hbox1, &wch->w, 0);
	mbtk_checkbox_style_set_prop(wch, mbtk_kw("fg.color"), mbtk_arg_str("#ff0000")); /* change the foreground color, which is the color of the mark */


	/*** Separator ***/
	wsep = mbtk_hsep_new(NULL);
	mbtk_box_add_widget(vbox, &wsep->w, 0);


	/*** Radio button ***/

	/* create and append a hbox; this will host our example radio buttons */
	hbox1 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	/* create a radio handler within the current widow */
	hrr = mbtk_handler_radio_alloc(win);

	/* create a radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 1");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_radio_add_widget(hrr, &wrb->w); /* add to the radio handler; this groups our radio buttons */

	/* create a second radio button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 2");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_handler_radio_add_widget(hrr, &wrb->w);


	/* create a third button.... */
	wrb = mbtk_radiobutton_new(NULL);
	mbtk_radiobutton_callback_toggle(wrb, button_toggle, "radio 3");
	mbtk_box_add_widget(hbox1, &wrb->w, 0);
	mbtk_radiobutton_style_set_prop(wrb, mbtk_kw("fg.color"), mbtk_arg_str("#ff0000")); /* change the foreground color, which is the color of the mark */
	mbtk_handler_radio_add_widget(hrr, &wrb->w);

	/*** canvas ***/
	wcs = mbtk_canvas_new(NULL);
	mbtk_canvas_set_minsize_cb(wcs, canvas_minsize);
	mbtk_canvas_set_draw_cb(wcs, canvas_draw);
	mbtk_box_add_widget(vbox, &wcs->w, 0);
	mbtk_canvas_callback_event(wcs, canvas_ev, "CaNvAs");


	/*** progress bar ***/
	wprb = mbtk_progbar_new(NULL);
	mbtk_box_add_widget(vbox, &wprb->w, 0);
	wprb->curr = 0.33;

	/*** scroll bar ***/
	wscb = mbtk_scrollbar_new(NULL, 1);
/*	mbtk_scrollbar_set_minsize(wscb, 16, 100);*/
	mbtk_box_add_widget(vbox, &wscb->w, 0);
	wscb->curr = 0.33;
	mbtk_scrollbar_callback_range_moved(wscb, range_moved, NULL);

	/*** picture ***/
	pickw = mbtk_kw_uniq();
	if (mbtk_xpm_set(&disp, pickw, xpm_test) != 0) {
		fprintf(stderr, "Failed to import pixmap\n");
		return -1;
	}
	wpic = mbtk_picture_new(NULL, pickw);
	mbtk_box_add_widget(vbox, &wpic->w, 0);
	mbtk_picture_style_set_prop(wpic, mbtk_kw("bg.color"), mbtk_arg_str("#00ff00"));



	/*** set alignment/layout ***/
/*
	mbtk_box_add_widget(vbox, &w2->w, 0);
	mbtk_picture_style_set_prop(w2, mbtk_kw("bg.color"), mbtk_arg_str("#00ff0010"));
*/


	mbtk_box_set_align(vbox, HVBOX_JUSTIFY);
	mbtk_box_set_spacing(vbox, 4);
/*	mbtk_box_set_span(w1, 20);*/
	mbtk_box_set_span(w2, HVBOX_FILL);

	mbtk_box_set_padding(vbox, 14);
	mbtk_box_set_padding(w1, 14);
	mbtk_box_set_padding(w2, 14);



	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_iterate(&disp, &ev, 1)) {
/*			printf("ev %d\n", ev.type);*/
			if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;

				if (ev.data.key.sym == 'p') {
					mbtk_arg_t a[4] = {0};
					mbtk_widget_get_propa(&wprb->w, mbtk_kw("range.val"), a, 4);
					printf("prog: %.2f %.2f   %.2f %.2f\n", a[0].v.d, a[1].v.d, a[2].v.d, a[3].v.d);
					a[2].v.d += 0.1;
					mbtk_widget_set_propa(&wprb->w, mbtk_kw("range.val"), a, 4);
				}

				if (ev.data.key.sym == 'o') {
					mbtk_arg_t a[4] = {0};
					mbtk_widget_get_propa(&wprb->w, mbtk_kw("range.coords"), a, 4);
					printf("prog: %d %d   %d %d\n", a[0].v.c.x, a[1].v.c.x, a[2].v.c.x, a[3].v.c.x);
					a[2].v.c.x += 5;
					mbtk_widget_set_propa(&wprb->w, mbtk_kw("range.coords"), a, 4);
				}
				if (ev.data.key.sym == 'l') {
					mbtk_progbar_set_curr(wprb, 0.2, 0);
				}
			}
		}
	}

	mbtk_uninit();
}
